@@ -34,7 +34,7 @@ class Client(AbstractAsyncClient):
         """
 
         kwargs = {
-            'headers': {'Content-Type': 'application/json'},
+            'headers': {'Content-Type': pjrpc.common.DEFAULT_CONTENT_TYPE},
             **kwargs,
         }
 
@@ -46,7 +46,7 @@ class Client(AbstractAsyncClient):
             return None
 
         content_type = resp.headers.get('Content-Type', '')
-        if response_text and content_type.split(';')[0] != 'application/json':
+        if response_text and content_type.split(';')[0] not in pjrpc.common.RESPONSE_CONTENT_TYPES:
             raise pjrpc.exc.DeserializationError(f"unexpected response content type: {content_type}")
 
         return response_text
