@@ -108,7 +108,7 @@ class Application:
         methods = {path: dispatcher.registry.values() for path, dispatcher in self._endpoints.items()}
         schema = self._spec.schema(path=endpoint_path, methods_map=methods)
 
-        return web.json_response(text=json.dumps(schema, indent=2, cls=specs.JSONEncoder))
+        return web.json_response(text=json.dumps(schema, cls=specs.JSONEncoder))
 
     async def _ui_index_page(self, request: web.Request) -> web.Response:
         app_path = request.path.rsplit(self._spec.ui_path, maxsplit=1)[0]
@@ -127,7 +127,7 @@ class Application:
         :returns: :py:class:`aiohttp.web.Request`
         """
 
-        if http_request.content_type != 'application/json':
+        if http_request.content_type not in pjrpc.common.REQUEST_CONTENT_TYPES:
             raise web.HTTPUnsupportedMediaType()
 
         try:
