@@ -1,9 +1,10 @@
 import dataclasses as dc
 import inspect
 import itertools as it
-from typing import Any, Callable, Dict, Iterable, List, Union
+from typing import Any, Callable, Dict, Iterable, List, Optional, Union
 
 from pjrpc.common import UNSET, UnsetType
+from pjrpc.common.exceptions import JsonRpcError
 
 
 @dc.dataclass(frozen=True)
@@ -33,6 +34,19 @@ class Example:
     description: str = UNSET
 
 
+@dc.dataclass(frozen=True)
+class ErrorExample:
+    """
+    Method error example.
+    """
+
+    code: int
+    message: str
+    data: Optional[Any] = UNSET
+    summary: str = UNSET
+    description: str = UNSET
+
+
 @dc.dataclass(frozen=True)
 class Tag:
     """
@@ -53,6 +67,11 @@ class Error:
     code: int
     message: str
     data: Dict[str, Any] = UNSET
+    data_required: bool = UNSET
+    title: str = UNSET
+    description: str = UNSET
+    deprecated: bool = UNSET
+    definitions: Dict[str, Any] = UNSET
 
 
 class BaseSchemaExtractor:
@@ -100,7 +119,11 @@ class BaseSchemaExtractor:
 
         return summary
 
-    def extract_errors_schema(self, method: Callable) -> Union[UnsetType, List[Error]]:
+    def extract_errors_schema(
+        self,
+        method: Callable,
+        errors: Optional[Iterable[JsonRpcError]] = None,
+    ) -> Union[UnsetType, List[Error]]:
         """
         Extracts method errors schema.
         """
@@ -121,6 +144,20 @@ class BaseSchemaExtractor:
 
         return UNSET
 
+    def extract_error_examples(
+        self,
+        method: Callable,
+        errors: Optional[Iterable[JsonRpcError]] = None,
+    ) -> Union[UnsetType, List[ErrorExample]]:
+        """
+        Extracts method error examples.
+        """
+
+        return [
+            ErrorExample(code=error.code, message=error.message, summary=error.message)
+            for error in errors
+        ] if errors else UNSET
+
     def extract_deprecation_status(self, method: Callable) -> Union[UnsetType, bool]:
         """
         Extracts method deprecation status.
